import { LanguageSupport } from "@codemirror/language";
import { markdocLinterExtension } from "./markdoc-linter-extension.mjs";
import { html } from "@codemirror/lang-html";
import { liquid, closePercentBrace } from "@codemirror/lang-liquid";
import { liquidCompletionSource } from "./liquid-completion-patch.mjs";
const markdoc = ({
  markdoc: markdocConfig = {},
  completion: liquidCompletion = {}
} = {}) => {
  let base = html();
  let liqu = liquid({ base });
  let lang = liqu.language;
  return new LanguageSupport(lang, [
    lang.data.of({
      autocomplete: liquidCompletionSource({
        tags: [
          ...liquidCompletion.tags ?? [],
          ...Object.keys(markdocConfig.tags ?? {}).map((s) => ({
            label: s,
            info: () => schemaToDomCompletionTooltip((markdocConfig.tags ?? {})[s])
          }))
        ],
        variables: [
          ...liquidCompletion.variables ?? [],
          ...Object.keys(markdocConfig.variables ?? {}).map((s) => ({
            label: s
          }))
        ],
        filters: liquidCompletion.filters,
        // no filter in markdoc
        properties: liquidCompletion.properties
      })
    }),
    base.language.data.of({ closeBrackets: { brackets: ["{"] } }),
    closePercentBrace,
    markdocLinterExtension(markdocConfig)
  ]);
};
function schemaToDomCompletionTooltip(schema = {}) {
  const div = document.createElement("div");
  const attrs = schema.attributes ?? {};
  div.innerHTML = Object.keys(attrs).map((k) => {
    const a = attrs[k];
    const type_ = Array.isArray(a.matches) && a.matches.every((i) => typeof i === "string") ? `(${a.matches.map((s) => JSON.stringify(s)).join(" | ")})` : typeof a.type === "string" ? a.type.toLowerCase() : "any";
    const require_ = a.required === true ? "" : "?";
    const default_ = typeof a.default === "string" || typeof a.default == "number" || typeof a.default === "boolean" ? ` = ${JSON.stringify(a.default)}` : "";
    return `(property) <b>${k}</b>${require_}: ${type_}${default_}`;
  }).join("<br>");
  return div;
}
export {
  markdoc
};
